#!/bin/bash
#
# Federated Computer Control Postgresql Service

PATH=$HOME/.docker/cli-plugins:/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin

config_postgresql() {
  echo -ne "\n* Configuring fstack/postgresql container.."
  spin &
  SPINPID=$!

  if [ ! -d "fstack/postgresql" ]; then
    mkdir -p fstack/postgresql/data/var/lib/postgresql fstack/postgresql/data/docker-entrypoint-initdb.d
    cp fstack/dns/data/etc/letsencrypt/archive/$DOMAIN/fullchain1.pem fstack/postgresql/data/var/lib/postgresql/server.crt
    cp fstack/dns/data/etc/letsencrypt/archive/$DOMAIN/privkey1.pem fstack/postgresql/data/var/lib/postgresql/server.key
    chown 999 fstack/postgresql/data/var/lib/postgresql/server.*
    chmod 600 fstack/postgresql/data/var/lib/postgresql/server.*
  fi

  DOMAIN_ARRAY=(${DOMAIN//./ })
  DOMAIN_FIRST=${DOMAIN_ARRAY[0]}
  DOMAIN_LAST=${DOMAIN_ARRAY[1]}

cat > fstack/postgresql/docker-compose.yml <<EOF
version: "3.7"

services:
  postgresql:
    image: postgres:14
    container_name: postgresql
    hostname: postgresql.$DOMAIN
    domainname: $DOMAIN
    restart: always
    networks:
      fstack:
        ipv4_address: 172.99.0.40
    volumes:
      - ./data/var/lib/postgresql/server.crt:/var/lib/postgresql/server.crt
      - ./data/var/lib/postgresql/server.key:/var/lib/postgresql/server.key
      - ./data/var/lib/postgresql/data:/var/lib/postgresql/data
    environment:
      - POSTGRES_DB=nextcloud
      - POSTGRES_USER=nextcloud
      - POSTGRES_PASSWORD=$ADMINPASS
    command: >
      -c ssl=on
      -c ssl_cert_file=/var/lib/postgresql/server.crt
      -c ssl_key_file=/var/lib/postgresql/server.key
    healthcheck:
      test: ["CMD-SHELL", "pg_isready -U nextcloud"]
      interval: 10s
      timeout: 5s
      retries: 5

networks:
  fstack:
    external: true
EOF

# cat postgresql/data/docker-entrypoint-initdb.d/init.sql
cat > fstack/postgresql/data/docker-entrypoint-initdb.d/init.sql <<EOF
CREATE USER vaultwarden WITH PASSWORD '$ADMINPASS';
CREATE DATABASE vaultwarden;
GRANT ALL PRIVILEGES ON DATABASE vaultwarden TO vaultwarden;
CREATE USER listmonk WITH PASSWORD '$ADMINPASS';
CREATE DATABASE listmonk;
GRANT ALL PRIVILEGES ON DATABASE listmonk TO listmonk;
EOF
 
kill -9 $SPINPID &> /dev/null
echo -ne "done."
}

start_postgresql() {
  # Start fstack/postgresql with output to /dev/null
  echo -ne "\n* Starting fstack/postgresql service.."
  spin &
  SPINPID=$!

  if [ $DEBUG ]; then
    # Start fstack/postgresql with output to console for debug
    docker-compose -f fstack/postgresql/docker-compose.yml -p postgresql up --build
    [ $? -eq 0 ] && echo -ne "done.\n" || fail "There was a problem starting service fstack/postgresql"
  else
    docker-compose -f fstack/postgresql/docker-compose.yml -p postgresql up --build -d &> /dev/null

    # Keep trying postgresql port 5432 to make sure it's up
    # before we proceed
    RETRY="30"
    while [ $RETRY -gt 0 ]; do
      nc -z 172.99.0.40 5432 &> /dev/null
      if [ $? -eq 0 ]; then
        break
      else
        if [ "$RETRY" == 1 ]; then
          docker-compose -f fstack/postgresql/docker-compose.yml -p postgresql down &> /dev/null
          kill -9 $SPINPID &> /dev/null
          fail "There was a problem starting service fstack/postgresql\nCheck the output of 'docker logs postgresql' or turn on\ndebug with -d"
        fi
        ((RETRY--))
        sleep 7
      fi
    done
  fi

  kill -9 $SPINPID &> /dev/null
  echo -ne "done."
}
